# -*- coding: utf-8 -*-
# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.
#
#  Licensed to Elasticsearch B.V. under one or more contributor
#  license agreements. See the NOTICE file distributed with
#  this work for additional information regarding copyright
#  ownership. Elasticsearch B.V. licenses this file to you under
#  the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.


from typing import Any, List

from ..exceptions import OpenSearchException


class BulkIndexError(OpenSearchException):
    @property
    def errors(self) -> List[Any]:
        """List of errors from execution of the last chunk."""
        return self.args[1]  # type: ignore


class ScanError(OpenSearchException):
    scroll_id: str

    def __init__(self, scroll_id: str, *args: Any, **kwargs: Any) -> None:
        super(ScanError, self).__init__(*args, **kwargs)
        self.scroll_id = scroll_id
